package inventorymanager

import (
	"context"
	"time"

	tpb "code.fbi.h-da.de/danet/gosdn/api/go/gosdn/transport"
	"code.fbi.h-da.de/danet/gosdn/applications/inventory-manager/config"
	"code.fbi.h-da.de/danet/gosdn/controller/api"
	"github.com/google/uuid"
	"github.com/sirupsen/logrus"
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
)

type InventoryManager struct {
	pndID             string
	controllerAddress string
	grpcClientConn    *grpc.ClientConn
	sessionContext    context.Context
	sessionToken      string
	//eventService          event.ServiceInterface
	configNetworkElements []config.NetworkElement // NetworkElements from the config file
}

// InitializeInventoryManager sets up the inventory manager and logs in to the controller.
func InitializeInventoryManager(config *config.Config) (*InventoryManager, error) {
	// Create controller client object
	newConn, err := grpc.NewClient(config.ControllerAddress, grpc.WithTransportCredentials(insecure.NewCredentials()))
	if err != nil {
		logrus.Errorf("Failed to create controller client with error: %s", err.Error())
		return nil, err
	}

	var conn *grpc.ClientConn
	var sessionToken string
	logrus.Info("Connecting and logging in to controller ", config.ControllerAddress)
	for {
		sessionToken, err = Login(conn, config.ControllerAddress, config.UserName, config.UserPW)
		if err == nil {
			logrus.Info("Connected and logged in to Controller")
			conn = newConn
			break
		}
		logrus.Errorf("Failed to connect and log in to Controller with error: %s, retrying in 1 second", err.Error())
		time.Sleep(1 * time.Second)
	}

	app, err := NewInventoryManager(conn, sessionToken, config.ControllerAddress, config.PndID, config.AppName, config.NetworkElements)
	if err != nil {
		return nil, err
	}

	return app, nil
}

func NewInventoryManager(grpcClientConn *grpc.ClientConn, sessionToken, controllerAddress, pndID, appName string, networkElements []config.NetworkElement) (*InventoryManager, error) {
	ctx := createContextWithAuthorization(sessionToken)

	// Not needed now, but later when lifecycle is implemented
	//queueCredentials, err := getQueueCredentials(ctx, controllerAddress, appName, registrationToken)
	//if err != nil {
	//	return nil, err
	//}
	//
	//eventService, err := event.NewEventService(
	//	queueCredentials,
	//	[]event.Topic{event.ManagedNetworkElement},
	//)
	//if err != nil {
	//	return nil, err
	//}

	return &InventoryManager{
		grpcClientConn: grpcClientConn,
		sessionContext: ctx,
		sessionToken:   sessionToken,
		//eventService:          eventService,
		pndID:                 pndID,
		controllerAddress:     controllerAddress,
		configNetworkElements: networkElements,
	}, nil
}

// Run starts the InventoryManager. This will elad to it trying to create every network element from the config file.
// It will try to create it indefinitely.
// TODO: After creating a network element, it will ensure that the element exists in the controller.
func (i *InventoryManager) Run() error {
	i.createNetworkelements()
	logrus.Info("Created all network elements.")
	for {
		// Management logic after creation should replace this sleep
		time.Sleep(60 * time.Second)
	}
}

func (i *InventoryManager) createNetworkelements() {
IterateDevicesLoop:
	for _, networkElement := range i.configNetworkElements {
		transportOptions := tpb.TransportOption{
			Address:  networkElement.Address,
			Username: networkElement.Username,
			Password: networkElement.Password,
			Tls:      networkElement.Tls,
			TransportOption: &tpb.TransportOption_GnmiTransportOption{
				GnmiTransportOption: &tpb.GnmiTransportOption{},
			},
		}

	AddDeviceLoop:
		for {
			pluginUUID, err := uuid.Parse(networkElement.PluginID)
			if err != nil {
				logrus.Errorf("No or wrong plugin UUID for device %s provided. Skipping device creation.", networkElement.Name)
				continue IterateDevicesLoop
			}
			pndUUID, err := uuid.Parse(i.pndID)
			if err != nil {
				logrus.Errorf("No or wrong PND UUID for device %s provided. Skipping device creation.", networkElement.Name)
				continue IterateDevicesLoop
			}

			_, err = api.AddNetworkElement(i.sessionContext, i.controllerAddress, networkElement.Name, networkElement.UUID, &transportOptions, pluginUUID, pndUUID, []string{})
			if err != nil {
				logrus.Errorf("Failed to create network element %s with error: %s. Sleeping one second and retrying...", networkElement.Name, err.Error())
				time.Sleep(1 * time.Second)
				continue AddDeviceLoop
			}
			break AddDeviceLoop
		}
		logrus.Infof("Successfully created network element %s", networkElement.Name)
	}
}
