package main

import (
	"flag"
	"log"
	"os"

	"code.fbi.h-da.de/danet/gosdn/applications/inventory-manager/config"
	inventorymanager "code.fbi.h-da.de/danet/gosdn/applications/inventory-manager/inventoryManager"
	"github.com/sirupsen/logrus"
	"google.golang.org/grpc/resolver"
	"gopkg.in/yaml.v3"
)

func main() {
	configFile := flag.String("config", "", "app config file")
	logLevel := flag.String("log", "info", "logrus lof level (debug, info, warn, error, fatal, panic)")
	noGRPCPassthrough := flag.Bool("noGRPCPassthrough", false, "set the default resolve scheme for grpc to not use  passthrough, default is false")

	flag.Parse()

	if *noGRPCPassthrough {
		logrus.Info("gRPC default resolver scheme is not set to passthrough. This might cause issues with the gRPC connection when no real DNS server is available as each gRPC requests requires a DNS request.")
	} else {
		logrus.Info("Setting gRPC default resolver scheme to passthrough. No DNS queries are being made when doing a gRPC request.")
		resolver.SetDefaultScheme("passthrough")
	}

	// parse string, this is built-in feature of logrus
	ll, err := logrus.ParseLevel(*logLevel)
	if err != nil {
		ll = logrus.InfoLevel
		logrus.Warn("invalid log level, using default: ", ll)
	}

	// set global log level
	logrus.Info("setting log level to ", ll)
	logrus.SetLevel(ll)

	logrus.Debugf("current config path: %s", *configFile)
	file, err := os.ReadFile(*configFile)
	if err != nil {
		currentFolder, _ := os.Getwd()
		log.Fatalf("error reading config file: %s, current folder: %s", *configFile, currentFolder)
	}

	appConfig := &config.Config{}
	if err = yaml.Unmarshal(file, appConfig); err != nil {
		logrus.Fatal(err)
	}

	// Just for printing the config
	configBytes, err := yaml.Marshal(appConfig)
	if err != nil {
		logrus.Fatal("error marshaling config to YAML: ", err)
	}
	logrus.Debugf("config:\n%s", string(configBytes))

	inventoryManager, err := inventorymanager.InitializeInventoryManager(appConfig)
	if err != nil {
		logrus.Fatal(err)
	}

	err = inventoryManager.Run()
	if err != nil {
		logrus.Fatal(err)
	}
}
